#include "tree.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <assert.h>

/*
 * Private Interface 
 */

struct Tree_t {
    char *item;     /* contained string */
    int count;      /* string count */
    Tree *left;     /* pointer to left-child */
    Tree *right;    /* pointer to right-child */
};

static Tree *make_node(const char *item)
/* Create a node containing string item, with left and right children NULL. */
{
    Tree *node = (Tree *)malloc(sizeof(Tree));
    if (node == NULL) 
        return NULL;
    
    node->item = (char *)malloc(strlen(item)+1); /* +1 for terminating \0 */
    if (node->item == NULL) {
        free(node);
        return NULL;
    }
    strcpy(node->item, item);

    node->count = 1;
    node->left = node->right = NULL;
    return node;
}

static void print_node(Tree *node)
/* Print word and word count for node */
{
    assert(node != NULL);
    printf("%s %d\n", node->item, node->count);
}

/*
 * Implementations for Public Interface 
 */

Tree *add_item(Tree *node, const char *item)
/* Search for whether item already exists in tree. If not, add it to first empty
 * node location (in lexicographical order), if found, increment word count. Perform
 * recursive descent for search, return pointer to current node. */
{
    int cmp;

    if (node == NULL) /* found empty tree location, add item */
        return make_node(item);

    /* Recursive comparison to put item in correct location. */
    cmp = strcmp(item, node->item);
    if (cmp < 0) 
        node->left = add_item(node->left, item);
    else if (cmp > 0)
        node->right = add_item(node->right, item);
    else
        ++node->count; /* item already in tree, increment count */

    return node;
}

int count_item(Tree *root, const char *item)
/* Search for item in tree and return its count. */
{
    while (root) {
        int cmp = strcmp(item, root->item);
        if (cmp < 0) 
            root = root->left;
        else if (cmp > 0)
            root = root->right;
        else 
             return root->count;
    }
    return 0;
}

void print_inorder(Tree *node)
/* Print tree in lexicographical order */
{
    if (node == NULL) return;
    print_inorder(node->left);
    print_node(node);
    print_inorder(node->right);
}

void destroy_tree(Tree *root)
/* Recursively free entire tree. */
{
    if (root == NULL) return;
    destroy_tree(root->left);
    destroy_tree(root->right);
    free(root->item);
    free(root);
}
